/* grid.h is part of VIP program
 * Written by J. Salvador Arias, 2010
 *      CONICET, INSUE, Instituto Miguel Lillo,
 *      Miguel Lillo 205, S.M. de Tucuman (4000)
 *      Tucuman, Argentina
 *      e-mail: jsalarias@csnat.unt.edu.ar
 *
 * VIP is a free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation and
 * Creative Commons.
 *
 * This file and the VIP program is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this file; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 * Or download it at:
 * http://creativecommons.org/licenses/GPL/2.0/
 */


#ifndef GRID_HEADER_INCLUDED
#define GRID_HEADER_INCLUDED

#include "ghead.h"
#include "bitfield.h"
#include "biogeo.h"

typedef struct tagQUADTREE QUADTREE;
struct tagQUADTREE {
 SINTEGER firstRow;
 SINTEGER lastRow;
 SINTEGER firstCol;
 SINTEGER lastCol;
 SCOUNTER cell;
 SINTEGER size;
 SFLAG state;
 SFLAG obsCell;
 GNode* node;
};

/* ------ Global variables ------ */
SCOUNTER numCells;
SCOUNTER numRows;
SCOUNTER numCols;

#define GETNUMCELLS numCells
#define GETNUMCOLS  numCols
#define GETNUMROWS  numRows

SREAL actualGridSize;
SREAL gridStartX;
SREAL gridStartY;
SCOUNTER actualFill;

#define GETGRIDSIZE   actualGridSize
#define GETGRIDSTARTX gridStartX
#define GETGRIDSTARTY gridStartY
#define GETFILL       actualFill

#define SETGRIDSIZE(x)   actualGridSize = (x)
#define SETFILL(x)       actualFill = (x)

/* ------ Functions ------ */
SCOUNTER GetCell (SINTEGER row, SINTEGER col);
void CreateGrid ();
void DestroyGrid ();

SFLAG GetGridState ();

void MapIntoQTree (BITFIELD* setA, BITFIELD* setB);
SFLAG GetRowColState (SINTEGER row, SINTEGER col);
void GetColRowFromCell (SCOUNTER cell, SINTEGER* row, SINTEGER* col);

SCOUNTER GetCellIfObs (SINTEGER row, SINTEGER col);

void FillWithReference (TERMINAL* term, BITFIELD* ref, BITFIELD* dest, SFLAG mode);
void GetObsSet (BITFIELD* obs, GNode* refNode);

#endif
